from .account import AccountClient
from .asset import AssetClient
from .base import Client
from .market import MarketClient
from .public import PublicClient
from .subaccount import SubAccountClient
from .system import SystemClient
from .trade import TradeClient



import pickle
from typing import Any

import redis
import six
from fastapi import APIRouter
from notebuild.tool.fastapi import api_route


class BaseConnect(APIRouter):
    def __init__(self, cache_prefix='cache', *args, **kwargs):
        self.cache_prefix = cache_prefix
        super(BaseConnect, self).__init__(*args, **kwargs)

    @api_route('/get', description="get value")
    def get_key(self, suffix=""):
        return f"{self.cache_prefix}_{suffix}"

    @api_route('/update', description="update value")
    def update_value(self, suffix=""):
        raise Exception("not implements")

    def get_value(self, suffix=""):
        raise Exception("not implements")

    def put_value(self, key, value: Any):
        raise Exception("not implements")


class SqlalchemyConnect(BaseConnect):
    def __init__(self, *args, **kwargs):
        super(SqlalchemyConnect, self).__init__(*args, **kwargs)



import logging
import os
logger = logging.getLogger()
logger.setLevel(logging.INFO)

from ccxt import okex
from notecoin.coins.base.file import DataFileProperty
from notecoin.coins.base.load import LoadDataKline
from notecoin.utils.time import day_during, month_during, week_during
from notefile.compress import tarfile



def load_and_save(file_pro: DataFileProperty):
    file_pro.sync()

    if file_pro.tar_exists():
        return

    exchan = LoadDataKline(file_pro.exchange)
    exchan.table.delete_all()
    unix_start, unix_end = int(file_pro.start_date.timestamp() * 1000), int(file_pro.end_date.timestamp() * 1000)
    # 下载
    exchan.load_all(timeframe=file_pro.timeframe, unix_start=unix_start, unix_end=unix_end)
    # 保存
    exchan.table.to_csv_all(file_pro.file_path_csv(), page_size=100000)
    # 压缩
    with tarfile.open(file_pro.file_path_tar(), "w:xz") as tar:
        tar.add(file_pro.file_path_csv(), arcname=file_pro.arcname(file_pro.file_path_csv()))
    # 删除
    os.remove(file_pro.file_path_csv())
    exchan.table.delete_all()


def load_monthly(file_pro: DataFileProperty):
    file_pro.freq = 'monthly'
    file_pro.file_format = '%Y%m'
    for index in range(1, 12*3):
        file_pro.start_date, file_pro.end_date = month_during(-index)
        load_and_save(file_pro)


def load_weekly(file_pro: DataFileProperty):
    file_pro.freq = 'weekly'
    file_pro.file_format = '%Y%m%d'
    for index in range(1, 54*3):
        file_pro.start_date, file_pro.end_date = week_during(-index)
        load_and_save(file_pro)


def load_daily(file_pro: DataFileProperty):
    file_pro.freq = 'daily'
    file_pro.file_format = '%Y%m%d'
    for index in range(1, 365*3):
        file_pro.start_date, file_pro.end_date = day_during(-index)
        load_and_save(file_pro)


path_root = '/home/bingtao/workspace/tmp'

file_pro = DataFileProperty(exchange=okex(), freq='daily', path=path_root, timeframe='1m')
# load_daily(file_pro)
# load_weekly(file_pro)
# file_pro.sync()
file_pro.load()



import logging
import os
from datetime import datetime

from notecoin.base.database.lanzou import LanzouDirectory
from notecoin.coins.base.load import LoadDataKline
from notecoin.utils.time import day_during, month_during, week_during
from notefile.compress import tarfile

logger = logging.getLogger()


class DataFileProperty:
    def __init__(self, exchange, data_type='kline', path='~/workspace/tmp', start_date=datetime.today(),
                 end_date=datetime.today(), freq='daily', timeframe='1m', file_format='%Y%m%d'):
        self.path = path
        self.freq = freq
        self.exchange = exchange
        self.data_type = data_type
        self.timeframe = timeframe
        self.start_date = start_date
        self.end_date = end_date
        self.file_format = file_format
        self.exchange_name = exchange.name.lower()
        self.drive = LanzouDirectory(fid=5679873)

    def file_path_dir(self, absolute=True):
        path = f"notecoin/{self.exchange_name}/{self.data_type}-{self.freq}-{self.timeframe}"
        if absolute:
            path = f"{self.path}/{path}"
            if not os.path.exists(path):
                os.makedirs(path)
        return path

    @property
    def filename_prefix(self):
        return f"{self.exchange_name}-{self.data_type}-{self.freq}-{self.timeframe}-{self.start_date.strftime(self.file_format)}"

    def file_path_csv(self, absolute=True):
        return f"{self.file_path_dir(absolute)}/{self.filename_prefix}.csv"

    def file_path_tar(self, absolute=True):
        return f"{self.file_path_dir(absolute)}/{self.filename_prefix}.tar"

    def arcname(self, file):
        return os.path.join(self.file_path_dir(False), os.path.basename(file))

    def sync(self):
        self.drive.scan_all_file()
        self.drive.sync(f'{self.path}/notecoin')

    def tar_exists(self):
        if self.drive.file_exist(self.file_path_tar(False)):
            return True
        if os.path.exists(self.file_path_tar()):
            return False

    def _load_and_save(self) -> bool:
        self.sync()
        if self.tar_exists():
            return False

        logger.info(f'download for {self.file_path_tar(absolute=False)}')
        exchan = LoadDataKline(self.exchange)
        exchan.table.delete_all()
        unix_start, unix_end = int(self.start_date.timestamp() * 1000), int(self.end_date.timestamp() * 1000)
        # 下载
        exchan.load_all(timeframe=self.timeframe, unix_start=unix_start, unix_end=unix_end)
        # 保存
        exchan.table.to_csv_all(self.file_path_csv(), page_size=100000)
        # 压缩
        with tarfile.open(self.file_path_tar(), "w:xz") as tar:
            tar.add(self.file_path_csv(), arcname=self.arcname(self.file_path_csv()))
        # 删除
        os.remove(self.file_path_csv())
        exchan.table.delete_all()
        return True

    def _load(self, start_end_fun):
        index = 1
        for i in range(1, 10000):
            self.start_date, self.end_date = start_end_fun(-i)
            if self._load_and_save():
                index += 1
            if index > 100:
                break

    def load(self):
        if self.freq == 'daily':
            self.file_format = '%Y%m%d'
            self._load(day_during)
        elif self.freq == 'weekly':
            self.file_format = '%Y%m%d'
            self._load(week_during)
        elif self.freq == 'monthly':
            self.file_format = '%Y%m'
            self._load(month_during)
